function result = parse_per()
%
% Use: result = parse_per
% This returns a struct with fields that are described in detail below.
%
% The data is organized as a long vector similar to the way the hk99 parsed data
% However, there are important differences, study and recall are all
% together and are indexed by the type of event.  The words are no longer an
% index into a nounpool, they are simply the word strings with isCorrect
% indicating whether this word was a correct recall for the trial.
% Also, Importantly, the key for the listType of event is as follows:
%
% Category Free Association - Intra and Extra list free association
% CFA_LIST = 0
% Temporal Free Recall - Free recall from a list of unrelated items
% TFR_LIST = 1
% Category Free Recall - Free recall from a semantically coherent list
% CFR_LIST = 2
% Final Free Recall - Free recall at the end of the experiment (complete
% free for all untimed? recall)
% FFR_LIST = 3
%
%
% block: 1-10, events that are not in a block are coded as -999
% isCorrect: 0,1 valid for FR_RESP types, tells if that response was a correct(meaning was in study list for this trial).
% list: 0-28 list number
% listType: what type of list it is. -999 for non list related events
% msRespTime: relative unix time to last response or beginning of trial
% msTime: global unix time
% recalled: for study items, whether they were recalled at test, invalid for everything but study events
% respList: what list that response came from
% respSerialPos: serial position of recall item regardless of which list it came from
% serialPos: serial position for study items
% subject: unique ID of that subject
% subjectList: list of unique subject IDs for looping over subjects
% type: key for that type of event,
% word: the word for that event
%
% Author: Adler Perrote


load('catFR_events')

c_subject = {};
c_type = {};
c_word = {};
for k=1:28326
    c_subject{k} = deblank(subject(k,:));
    c_type{k} = deblank(type(k,:));
    c_word{k} = deblank(word(k,:));
end

result.subject = c_subject;
result.type = c_type;
result.word = c_word;
result.nounpool = unique(result.word);
result.wordindex = zeros(1,length(result.word));
for k=1:length(result.word)
    result.wordindex(k) = find(ismember(result.nounpool, result.word(k)));
end
result.isCorrect = isCorrect;
result.block = block;
result.list = list;
result.listType = listType;
result.msRespTime = msRespTime;
result.msTime = msTime;
result.recalled = recalled;
result.respList = respList;
result.respSerialPos = respSerialPos;
result.serialPos = serialPos;
result.subjlist = unique(result.subject);
