function [theta_r psi_recall] = sim_recall(prm,LDA,psi,theta_s)
% Simulate recall phase. The initial context is the last context from study.
% Authors: Richard Socher, Adler Perotte

[K W] = size(LDA.beta); %word distribution matrix

%parameters
poisson_lambda = prm.poisson_lambda;
lambda = prm.lambda;
eta2 = prm.eta2;
eta3 = prm.eta3;
eps1 = prm.eps1;
%eps2 = prm.eps2;
gamma = prm.gamma;
Sigma_r = prm.sigma*eye(K);

T = poissrnd(poisson_lambda);

sbeta = softmax(LDA.beta');
stheta_s = softmax(theta_s);

psi_lda_numbers = getLDAWordNumbers(psi,LDA.vocab);

%list of recalled words, to initialize \tilde{p}_o^r, we use the last study word for programming convenience
recalled_word_numbers = [];

%initial context
theta_r(:,1) = theta_s(:,end);

%uniform topic prior probabilities in mean parameter space
pr = ones(K,1)./K;

%loop through the number of words, you want to recall
for t = 1:T
    
    stheta_r = softmax(theta_r(:,t));
    delta_s = zeros(W,1);
    for j = 1:length(psi_lda_numbers)
        if ~ismember(psi_lda_numbers(j),recalled_word_numbers(1:end))
            delta_s(psi_lda_numbers(j)) = 1./(distance2(stheta_r, stheta_s(:,j))^eps1);% + 1./distance2(stheta_r, stheta_s(:,j));
        end
    end
    
    if sum(delta_s)>0
        delta_s = delta_s./sum(delta_s);
        phi = lambda*(sbeta*softmax(theta_r(:,t))) + (1-lambda)*delta_s;
    else
        disp(['all deltas are zero, during recall of word number ' num2str(t)])
        phi = (sbeta*softmax(theta_r(:,t)));
    end
    
    if abs(sum(phi)-1)<0.00000001
        sampled_word_number = randsample(1:W,1,true,phi);
    else
        error('multinomial parameters do not sum to 1')
    end
    
    recalled_word_numbers(end+1) = sampled_word_number;
    
    
    p_r = exp(LDA.beta(:,recalled_word_numbers(end)));
    p_r = p_r +  eps;                                 %avoid -\infty if prob of a word is 0...
    p_r = (p_r ./ sum(p_r)) .* pr;                    %normalize
    
    %sum of weighted study thetas
    stheta_r = theta_r(:,t);%softmax(theta_r(:,t-1));
    match_ind = psi_lda_numbers == recalled_word_numbers(end);
    if(~any(match_ind))
        recall_theta_study = (eta2/(eta2+eta3))*theta_r(:,t) + (eta3/(eta2+eta3))*log(p_r);
    else
        recall_theta_study = theta_s(:,match_ind);
    end
    
    %new mean of drifting context, see Eq. (2) in the report
    h = eta2*theta_r(:,t) + eta3*log(p_r) + (1-eta2-eta3)*recall_theta_study;
    theta_r(:,t+1) = mvnrnd(h,Sigma_r);               %sample new context
    
    
    if sum(exp(theta_r(:,t+1))) == 0
        error('exp(theta) = 0.')
    end
    
    if ~(sum(theta_r(:,t+1)) > -Inf)
        error('Theta = -Inf.')
    end
    
end

psi_recall = LDA.vocab(recalled_word_numbers(1:end));
